package no.ntnu.idatt1002.demo.data.Economics;

import java.util.ArrayList;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * ItemRegister is a generic class used by ExpenseRegister and
 * IncomeRegister.
 * @param <T>   Income or Expense
 */
public class ItemRegister<T>{
    ArrayList<T> items;

    /**
     * An "empty" class constructor.
     */
    public ItemRegister() {
        this.items = new ArrayList<>();   //ArrayList for storing T´s
    }

    /**
     * Class constructor that takes in an ArrayList of T as argument.
     * @param items  An ArrayList of Income or Expense you want to overview.
     */
    public ItemRegister(ArrayList<T> items) {
        this.items = items;
    }

    /**
     * Get an ArrayList of every T.
     * @return  T ArrayList.
     */
    public ArrayList<T> getItems() {
        return items;
    }

    /**
     * Add a new T to the register. Throws IllegalArgumentException
     * if the new T is already registered.
     * @param newItem   The T you want to add.
     */
    public void addItem(T newItem){
        if(items.contains(newItem)){
            throw new IllegalArgumentException("This item is already registered");
        }
        items.add(newItem);
    }

    /**
     * Get the sum of all T´s in items.
     * @return   Sum of all T´s.
     */
    public double getTotalSum(){
        ArrayList<Item> castedItems = (ArrayList<Item>) items; //Casts items as an ArrayList of Item´s
        return castedItems.stream().map(Item::getAmount).mapToDouble(Double::doubleValue).sum();
    }

    public <S> ArrayList<T> getItemsBasedOnCategory(S Category){
        if(Category instanceof IncomeCategory){ //Checks if it´s an instance of IncomeCategory
            ArrayList<Income> castedIncome = (ArrayList<Income>) items;
            return (ArrayList<T>) castedIncome.stream().filter(anIncome -> anIncome.getCategory().compareTo((IncomeCategory) Category) == 0).collect(Collectors.toList());
        } else if(Category instanceof ExpenseCategory){ //Checks if it´s an instance of ExpenseCategory
            ArrayList<Expense> castedExpenses = (ArrayList<Expense>) items;
            return (ArrayList<T>) castedExpenses.stream().filter(anExpense -> anExpense.getCategory().compareTo((ExpenseCategory) Category) == 0).collect(Collectors.toList());
        } else{return null;}
    }
}
